"""Miscellaneous small utility functions.

    vol(vol=None) - Get or set volume using aumix.

    progress(ratio, length=40, col=1, cols=("yellow", None, "cyan"),
            nocol="=.")
        Text mode progress bar.

    yes(question, [default answer]) 
        i.e. if yes("erase file?", 'n'): erase_file()

    color(text, fg, [bg])
        Colorize text using terminal color codes.

    beep(times) - Beep terminal bell number of times.

    ftime(seconds) - returns h:m:s or m:s if there's no hours.

    Term() - terminal stuff
        term.size() => height, width
        term.clear() => clear terminal
        term.getch() => get one char at a time

Andrei Kulakov <ak@silmarill.org>
"""

import os
import sys
try:
    import termios
except ImportError:
    import TERMIOS
    termios = TERMIOS
import commands
import string
from types import *

dbg = 1
enable_color = 0
hotkeycol = "red"

colors = {
    "black"     :   "30",
    "red"       :   "31",
    "green"     :   "32",
    "brown"     :   "33",
    "blue"      :   "34",
    "purple"    :   "35",
    "cyan"      :   "36",
    "lgray"     :   "37",
    "gray"      :   "1;30",
    "lred"      :   "1;31",
    "lgreen"    :   "1;32",
    "yellow"    :   "1;33",
    "lblue"     :   "1;34",
    "pink"      :   "1;35",
    "lcyan"     :   "1;36",
    "white"     :   "1;37"
    }


def debug(*msgs):
    lead = '### DEBUG ###  '
    if dbg:
        if type(msgs) == ListType:
            for m in msgs:
                print lead, m
        else:
            print lead, msgs
    raw_input("Hit enter..\n")


def split(fname):
    """Split filename into (name, extension) tuple."""

    tmp = fname.split('.')
    if len(tmp) == 1:
        # no extension
        return fname, None
    else:
        return string.join(tmp[:-1], '.'), tmp[-1]


def vol(vol=None):
    """Set or show audio volume.

    Uses external mixer called aumix. One optional argument, vol, may
    be an int or a string. If a string, it can be of the form "+10".
    """

    if vol:
        vol = str(vol)
        os.system("aumix -v" + vol)
    else:
        v = commands.getoutput("aumix -vq").split()[1]
        return v[:-1]


def progress(ratio, length=40, col=1, cols=("yellow", None, "cyan"),
            nocol="=."):
    """Text mode progress bar.

    ratio   - current position / total (e.g. 0.6 is 60%)
    length  - bar size
    col     - color bar
    cols    - tuple: (elapsed, left, percentage num)
    nocol   - string, if default="=.", bar is [=======.....]
    """

    # TODO: percent display in the middle of the bar
    if ratio > 1:
        ratio = 1
    elchar, leftchar = nocol
    elapsed = int(round(ratio*length))
    left = length - elapsed
    bar = elchar*elapsed + leftchar*left
    bar = bar[:length]
    if col:
        c_elapsed, c_left, perc = cols
        bar = color(' '*elapsed, "gray", c_elapsed)
        bar = bar + color(' '*left, "gray", c_left)
    else:
        bar = elchar*elapsed + leftchar*left
    return bar


def yes(question, default=None):
    """Get an answer for the question.

    Return 1 on 'yes' and 0 on 'no'; default may be set to 'y' or 'n';
    asks "Question? [Y/n]" (default is capitalized). Yy and Nn are
    acceptable. Question is asked until a valid answer is given.
    """

    y = 'y'
    n = 'n'
    if default:
        if default in "Yy":
            y = 'Y'
        elif default in "Nn":
            n = 'N'
        else:
            print "Error: default must be 'y' or 'n'."
            return
    while 1:
        answer = raw_input(question + " [%s/%s] " % (y, n))
        if default:
            if default in "Yy":
                if not answer or (len(answer) == 1 and answer in "Yy"):
                    return 1
                elif len(answer) == 1 and answer in "Nn":
                    return 0
            elif default in "Nn":
                if not answer or (len(answer) == 1 and answer in "Nn"):
                    return 0
                elif len(answer) == 1 and answer in "Yy":
                    return 1
        else:
            if len(answer) == 1 and answer in "Yy":
                return 1
            elif len(answer) == 1 and answer in "Nn":
                return 0

def no(question, default=None):
    return not yes(question, default)

def color(text, fg, bg=None, raw=0):
    """Return colored text.

    Uses terminal color codes; set avk_util.enable_color to 0 to
    return plain un-colored text. If fg is a tuple, it's assumed to
    be (fg, bg). Both colors may be 'None'.

    Raw means return string in raw form - for writing to a file instead of
    printing to screen. Leave default if not sure.
    """

    if not enable_color:
        return text
    if type(fg) in (TupleType, ListType):
        fg, bg = fg
    if not fg:
        return text
    opencol = "\033["
    closecol = "m"
    if raw:
        opencol = r"\[\033["
        closecol = r"m\]"
    clear = opencol + "0" + closecol
    xterm = 0
    if os.environ["TERM"] == "xterm": 
        xterm = 1
    b = ''
    # In xterm, brown comes out as yellow..
    if xterm and fg == "yellow": 
        fg = "brown"
    f = opencol + colors[fg] + closecol
    if bg:
        if bg == "yellow" and xterm: 
            bg = "brown"
        try: 
            b = colors[bg].replace('3', '4', 1)
            b = opencol + b + closecol
        except KeyError: 
            pass
    return "%s%s%s%s" % (b, f, text, clear)

def beep(times, interval=1):
    """Beep terminal bell specified times with given interval in seconds
    (fractions are okay).
    """

    import time 
    for t in range(times):
        print '\a'
        time.sleep(interval)

def ftime(seconds):
    """Format time. 
    
    hour:min:sec if hours > 0; otherwise min:sec.
    """

    seconds = int(seconds)
    hours = seconds / (60*60)
    seconds = seconds - hours*60*60
    mins = seconds / 60
    secs = seconds - mins*60
    if int(hours):
        s = "%d:%02d:%02d" % (hours, mins, secs)
    else:
        s = "%d:%02d" % (mins, secs)
    return s

class Term:
    """Linux terminal management.
    
    clear   - calls os.system("clear")
    getch   - get one char at a time
    size    - return height, width of the terminal
    """

    def __init__(self):
        self.fd = sys.stdin.fileno()
        self.new_term = termios.tcgetattr(self.fd)
        self.old_term = termios.tcgetattr(self.fd)
        self.ver = None
        if sys.version_info[0] == 2 and sys.version_info[1] == 0:
            self.ver = 20
        if self.ver == 20:
            self.new_term[3] = (self.new_term[3] & ~TERMIOS.ICANON & 
                ~TERMIOS.ECHO)
        else:
            self.new_term[3] = (self.new_term[3] & ~termios.ICANON & 
                ~termios.ECHO)

    def normal(self):
        """Set 'normal' terminal settings."""

        if self.ver == 20:
            termios.tcsetattr(self.fd, TERMIOS.TCSAFLUSH, self.old_term)
        else:
            termios.tcsetattr(self.fd, termios.TCSAFLUSH, self.old_term)

    def clear(self):
        """Clear screen."""

        os.system("clear")

    def cline(self):
        """Clear line."""

        width = self.size()[1]
        sys.stdout.write('\r' + ' '*width)

    def curses(self):
        """Set 'curses' terminal settings."""

        if self.ver == 20:
            termios.tcsetattr(self.fd, TERMIOS.TCSAFLUSH, self.new_term)
        else:
            termios.tcsetattr(self.fd, termios.TCSAFLUSH, self.new_term)

    def getch(self, prompt=None):
        """Get one character at a time.

        NOTE: if the user suspends (^Z) running program, then brings it
        back to foreground, you have to instantiate Term class again.
        Otherwise getch() won't work. Even after that, the user has to
        hit 'enter' once before he can enter commands.
        """

        if prompt:
            sys.stdout.write(prompt)
            sys.stdout.flush()
        self.curses()
        c = os.read(self.fd, 1)
        self.normal()
        return c

    def size(self):
        """Return terminal size as tuple (height, width)."""

        import struct, fcntl
        h,w = 0,0
        if self.ver == 20:
            h, w = struct.unpack("hhhh", fcntl.ioctl(0, 
                    TERMIOS.TIOCGWINSZ, "\000"*8))[0:2]
        else:
            h, w = struct.unpack("hhhh", fcntl.ioctl(0, 
                    termios.TIOCGWINSZ, "\000"*8))[0:2]
        if not h and w:
            h, w = 24, 80
        return h, w
